#include "stdafx.h"
#include "CCrossword.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CCrossword::CCrossword ()
{
	CellsListSize = 0;
	CellsList = NULL;
	
	Count ();
}

CCrossword::~CCrossword ()
{
	DeleteContent ();
}

void CCrossword::DeleteContent (void)
{
	CellsListSize = 0;
	if (CellsList != NULL)
	{	
		delete [] CellsList;
		CellsList = NULL;

		Count ();
	}
}

BOOL CCrossword::Load (CArchive& ar)
{
	int Type, X, Y, Length;
	CString Word;

	//   
	DeleteContent ();

	try
	{
		ar >> CellsListSize;
	}
	catch (CArchiveException *e)
	{
		//    
		e->Delete ();

		CellsListSize = 0;
		return FALSE;
	}
	if (CellsListSize < 0 || CellsListSize > MaxCrosswordSize)
	{
		//     
		CellsListSize = 0;
		return FALSE;
	}

	if (CellsListSize == 0)
		//      
		return TRUE;

	//    
	CellsList = new CCell [CellsListSize];
	if (CellsList == NULL)
		//  
		return FALSE;

	//  
	for (long i = 0; i < CellsListSize; i++)
	{
		try
		{
			ar >> Type;
			ar >> X;
			ar >> Y;
			ar >> Length;
			ar >> Word;
		}
		catch (CArchiveException *e)
		{
			//    
			e->Delete ();
			DeleteContent ();
			return FALSE;
		}

		if ((Type != Vertical && Type != Horizontal) ||
			(X < MinX) || (X > MaxX) || (Y < MinY) || (Y > MaxY) ||
			(Length < MinWordSize) || (Length > MaxWordSize) ||
			(Word.GetLength () != Length && Word.GetLength () != 0))
		{
			//  
			DeleteContent ();
			return FALSE;
		}

		CellsList [i].Set (Type, X, Y, Length, Word);
	}

	Link ();
	Count ();

	return TRUE;
}

BOOL CCrossword::Save (CArchive& ar)
{
	try
	{
		//   
		ar << CellsListSize;

		//  
		for (long i = 0; i < CellsListSize; i++)
		{
			CCell &cell = CellsList [i];
			ar << cell.Type;
			ar << cell.X;
			ar << cell.Y;
			ar << cell.Length;
			ar << cell.Word;
		}
	}
	catch (CArchiveException *e)
	{
		//    
		e->Delete ();
		return FALSE;
	}
		
	return TRUE;
}

BOOL CCrossword::TestAdd (int newType, int newX, int newY, int newLength)
{
	//   
	for (long i = 0; i < CellsListSize; i++)
	{
		CCell &cell = CellsList [i];

		//      
		//    
		if (newType == cell.Type)
		{
			if (newType == Horizontal)
			{
				if ((newY == cell.Y) && (newX + newLength > cell.X) &&
					(newX < cell.X + cell.Length))
					//   
					return FALSE;
			}
			else
			{
				if ((newX == cell.X) && (newY + newLength > cell.Y) &&
					(newY < cell.Y + cell.Length))
					//   
					return FALSE;
			}
		}
	}
	return TRUE;
}

BOOL CCrossword::Add (int newType, int newX, int newY, int newLength, LPCTSTR newWord)
{
	//   
	if (!TestAdd (newType, newX, newY, newLength))
		return FALSE;

	//      
	long newCellsListSize = CellsListSize + 1;
	CCell *newCellsList = new CCell [newCellsListSize];
	if (newCellsList == NULL)
		//  
		return FALSE;

	//     
	long i = 0;
	for (; i < CellsListSize; i++)
		newCellsList [i] = CellsList [i];

	//   
	newCellsList [i].Set (newType, newX, newY, newLength, newWord);
	
	//     
	DeleteContent ();
	CellsList = newCellsList;
	CellsListSize = newCellsListSize;
	
	Link ();
	Count ();

	return TRUE;
}

BOOL CCrossword::TestRemove (int Type, int X, int Y)
{
	//   
	for (long i = 0; i < CellsListSize; i++)
	{
		CCell &cell = CellsList [i];
		if ((cell.Type == Type) && (cell.X == X) && (cell.Y == Y))
			return TRUE;
	}
	
	//   
	return FALSE;
}

BOOL CCrossword::Remove (int Type, int X, int Y)
{
	//   
	if (!TestRemove (Type, X, Y))
		//   
		return FALSE;

	if (CellsListSize == 1)
	{
		//   
		DeleteContent ();
		return TRUE;
	}

	//      
	long newCellsListSize = CellsListSize - 1;
	CCell *newCellsList = new CCell [newCellsListSize];
	if (newCellsList == NULL)
		//  
		return FALSE;

	//      ( )
	for (long j = 0, i = 0; i < CellsListSize; i++)
	{
		CCell &cell = CellsList [i];
		if ((cell.Type != Type) || (cell.X != X) || (cell.Y != Y))
			newCellsList [j++] = cell;
	}

	//     
	DeleteContent ();
	CellsList = newCellsList;
	CellsListSize = newCellsListSize;
	
	Link ();
	Count ();

	return TRUE;
}

void CCrossword::Link (void)
{
	if (CellsList == NULL)
		//  
		return;

	int h, v;
	
	ClearLinks ();

	//   
	for (long hindex = 0; hindex < CellsListSize; hindex++)
	{
		CCell &hcell = CellsList [hindex];
		ASSERT (hcell.CrossList != NULL);

		//    
		if (hcell.Type == Horizontal)
		{
			//    (jindex)      (index)
			for (long vindex = 0; vindex < CellsListSize; vindex++)
			{
				CCell &vcell = CellsList [vindex];
				
				//       
				if ((vcell.Type == Vertical) &&
					(vcell.X >= hcell.X) &&
					(vcell.X < (hcell.X + hcell.Length)) &&
					(hcell.Y >= vcell.Y) &&
					(hcell.Y < (vcell.Y + vcell.Length)))
				{
					//      
					h = vcell.X - hcell.X;
					v = hcell.Y - vcell.Y;

					//  
					hcell.CrossList [h].CellNumber = vindex;
					hcell.CrossList [h].CharNumber = v;
					hcell.CrossCounter ++;
					vcell.CrossList [v].CellNumber = hindex;
					vcell.CrossList [v].CharNumber = h;
					vcell.CrossCounter ++;
				} // if
			} // for jindex
		} // if
	} // for index
}

void CCrossword::Sort (void)
{
	Link (); //    

	//      
	CCell &src = CellsList [0];
	for (long i = 1; i < CellsListSize; i++)
	{
		CCell &dest = CellsList [i];		
		if (src.CrossCounter < dest.CrossCounter)
//		if (dimDictionarySizesCounter [src.Length - MinWordSize] >
//			dimDictionarySizesCounter [dest.Length - MinWordSize])
		{
			CCell temp (src);
			src = dest;
			dest = temp;
		}
	}
	
	Link (); //    

	//   ,      
	//  ()
	long k = 0, j = 1;
	do
	{
		CCell &root = CellsList [k];
		
		//      root
		BOOL Founded = FALSE;
		for (long i = j; i < CellsListSize; i++)
		{
			if (IsLinked (root, i))
			{
				//    !
				if (i != j) //      - ...
				{
					CCell temp (CellsList [j]);
					CellsList [j] = CellsList [i];
					CellsList [i] = temp;
					
					Link (); //  
				}
				k = j; //  root  -  
				j++;
				Founded = TRUE;
				break;
			}
		}
		//     root   ,   
		if (!Founded)
		{
			k--;
			//    ,      
			if (k < 0)
			{
				k = j;
				j++;
			}
		}
	}
	//      
	while (j < CellsListSize - 1);

	//     - 
	Link ();	
}

BOOL CCrossword::IsLinked (CCell &src, long index)
{
	for (int i = 0; i < src.Length; i++)
		if (src.CrossList [i].CellNumber == index)
			return TRUE;
	return FALSE;
}

void CCrossword::Count (void)
{
	// 
	for (int j = 0; j < (MaxWordSize - MinWordSize + 1); j++)
		SizesCounter [j] = 0;

	// 
	for (long i = 0; i < CellsListSize; i++)
		SizesCounter [CellsList [i].Length - MinWordSize] ++;
}

void CCrossword::ClearLinks (void)
{
	//      
	for (long index = 0; index < CellsListSize; index++)
	{
		CellsList [index].ClearCrossList ();
		CellsList [index].CrossCounter = 0;
	}
}

void CCrossword::ClearChains (void)
{
	//      
	for (long index = 0; index < CellsListSize; index++)
	{
		CellsList [index].Word = "";
		CellsList [index].NumberOfWord = NullWord;
	}
}

void CCrossword::GetRect (CRect &rect)
{
	if (CellsListSize == 0)
	{
		//   
		rect.SetRect (0, 0, 0, 0);
		return;
	}

	//     
	CCell &firstcell = CellsList [0];
	rect.SetRect (firstcell.X, firstcell.Y,
		firstcell.X + ((firstcell.Type == Horizontal) ? firstcell.Length : 0),
		firstcell.Y + ((firstcell.Type == Vertical) ? firstcell.Length : 0));

	//   
	for (long index = 1; index < CellsListSize; index++)
	{
		CCell &cell = CellsList [index];
		if (rect.left > cell.X)
			rect.left = cell.X;
		if (rect.top > cell.Y)
			rect.top = cell.Y;
		if (rect.right < cell.X + ((cell.Type == Horizontal) ? cell.Length : 0))
			rect.right = cell.X + ((cell.Type == Horizontal) ? cell.Length : 0);
		if (rect.bottom < cell.Y + ((cell.Type == Vertical) ? cell.Length : 0))
			rect.bottom = cell.Y + ((cell.Type == Vertical) ? cell.Length : 0);
	}
}
