// CrossMasterDoc.cpp : implementation of the CCrossMasterDoc class
//
#include "stdafx.h"
#include "CrossMaster.h"
#include "MainFrm.h"
#include "CrossMasterDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//   
LPCTSTR Reg_Database			= _T("Database");
LPCTSTR Reg_DatabaseName		= _T("DatabaseName");
LPCTSTR Reg_WordsTableName		= _T("WordsTableName");
LPCTSTR Reg_WordsColumnName		= _T("WordsColumnName");
LPCTSTR Reg_WordKeysColumnName	= _T("WordKeysColumnName");
LPCTSTR Reg_QuestionColumnName	= _T("QuestionColumnName");
LPCTSTR Reg_TypesTableName		= _T("TypesTableName");
LPCTSTR Reg_TypesColumnName		= _T("TypesColumnName");
LPCTSTR Reg_TypeKeysColumnName	= _T("TypeKeysColumnName");

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterDoc

IMPLEMENT_DYNCREATE(CCrossMasterDoc, CDocument)

BEGIN_MESSAGE_MAP(CCrossMasterDoc, CDocument)
	//{{AFX_MSG_MAP(CCrossMasterDoc)
	ON_LBN_SELCHANGE(IDC_WORDS_LIST, OnSelchangeWordsList)
	ON_CBN_SELCHANGE(IDC_TYPES_LIST, OnSelchangeTypesList)
	ON_COMMAND(ID_CROSSWORD_START, OnCrosswordStart)
	ON_COMMAND(ID_CROSSWORD_STOP, OnCrosswordStop)
	ON_UPDATE_COMMAND_UI(ID_CROSSWORD_START, OnUpdateCrosswordStart)
	ON_UPDATE_COMMAND_UI(ID_CROSSWORD_STOP, OnUpdateCrosswordStop)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, OnUpdateFilePrintPreview)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_UPDATE_COMMAND_UI(ID_APP_EXIT, OnUpdateAppExit)
	ON_UPDATE_COMMAND_UI(ID_FILE_MRU_FILE1, OnUpdateFileMruFile1)
	ON_COMMAND(ID_FILE_ACCESS, OnFileAccess)
	ON_UPDATE_COMMAND_UI(ID_FILE_ACCESS, OnUpdateFileAccess)
	ON_COMMAND(ID_VIEW_CELL, OnViewCell)
	ON_UPDATE_COMMAND_UI(ID_VIEW_CELL, OnUpdateViewCell)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterDoc construction/destruction

CCrossMasterDoc::CCrossMasterDoc()
{
	//    
	m_pApp = (CCrossMasterApp *) AfxGetApp ();
	
	//    
	m_strDatabaseName =
		m_pApp->GetProfileString (Reg_Database, Reg_DatabaseName, "Words.mdb");
	m_strWordsTableName =
		m_pApp->GetProfileString (Reg_Database, Reg_WordsTableName, "WORDS");
	m_strWordsColumnName =
		m_pApp->GetProfileString (Reg_Database, Reg_WordsColumnName, "WORD");
	m_strWordKeysColumnName =
		m_pApp->GetProfileString (Reg_Database, Reg_WordKeysColumnName, "WORD_KEY");
	m_strQuestionColumnName =
		m_pApp->GetProfileString (Reg_Database, Reg_QuestionColumnName, "QUESTION");
	m_strTypesTableName =
		m_pApp->GetProfileString (Reg_Database, Reg_TypesTableName, "TYPES");
	m_strTypesColumnName =
		m_pApp->GetProfileString (Reg_Database, Reg_TypesColumnName, "TYPE");
	m_strTypeKeysColumnName =
		m_pApp->GetProfileString (Reg_Database, Reg_TypeKeysColumnName, "TYPE_KEY");

	//     
	m_bBuildResult = FALSE;
	m_bBuildInProgress = FALSE;
	m_bTerminateRequest = FALSE;

	//    
	m_dimTypes = NULL;
	m_bDictionaryLoaded = FALSE;
	m_nTypeSelect = 0;

	m_bEnableDrawCell = FALSE;
}

CCrossMasterDoc::~CCrossMasterDoc()
{
	//   
	CloseDatabase ();
	
	//    
	DeleteTypes ();

	//  
	m_pApp->WriteProfileString (Reg_Database, Reg_DatabaseName, m_strDatabaseName);	
	m_pApp->WriteProfileString (Reg_Database, Reg_WordsTableName, m_strWordsTableName);
	m_pApp->WriteProfileString (Reg_Database, Reg_WordsColumnName, m_strWordsColumnName);
	m_pApp->WriteProfileString (Reg_Database, Reg_WordKeysColumnName, m_strWordKeysColumnName);
	m_pApp->WriteProfileString (Reg_Database, Reg_QuestionColumnName, m_strQuestionColumnName);
	m_pApp->WriteProfileString (Reg_Database, Reg_TypesTableName, m_strTypesTableName);
	m_pApp->WriteProfileString (Reg_Database, Reg_TypesColumnName, m_strTypesColumnName);
	m_pApp->WriteProfileString (Reg_Database, Reg_TypeKeysColumnName, m_strTypeKeysColumnName);
}

BOOL CCrossMasterDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	ASSERT (m_bBuildInProgress == FALSE);

	return TRUE;
}

BOOL CCrossMasterDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (m_bBuildInProgress)
		//      
		return FALSE;

	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	
	return TRUE;
}

void CCrossMasterDoc::DeleteContents() 
{	
	ASSERT (m_bBuildInProgress == FALSE);

	//  
	Crossword.DeleteContent ();
	
	CDocument::DeleteContents();
}

BOOL CCrossMasterDoc::OpenDatabase (void)
{
	//     
	if (m_Database.IsOpen ())
		//  
		return TRUE;
	
	//        
	for (;;)
	{
		try
		{
			//      
			m_Database.Open (m_strDatabaseName, FALSE, TRUE);
			break;
		}
		catch (CDaoException* e)
		{
			// 
			e->Delete ();
		}
		//   
		CString msg;
		msg.Format ("%s\n\n   . :\n"
			"1. C  ,    \n"
			"2.     DAO 3.0\n", m_strDatabaseName);
		switch (AfxMessageBox (msg, MB_ABORTRETRYIGNORE | MB_ICONEXCLAMATION))
		{
			case IDABORT:
			//   
				return FALSE;
			
			case IDRETRY:
			//  
			break;

			case IDIGNORE:
			//     
				CFileDialog FileDlg (TRUE, NULL, m_strDatabaseName,
					OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_NOCHANGEDIR,
					" (*.mdb)|*.mdb||", AfxGetMainWnd ());
				FileDlg.m_ofn.lpstrTitle = " ";
				if (FileDlg.DoModal () == IDOK)
					//    
					m_strDatabaseName = FileDlg.GetPathName ();
			break;
		}
	}

	return TRUE;
}

void CCrossMasterDoc::CloseDatabase (void)
{
	//    
	if (m_Database.IsOpen ())
		m_Database.Close ();
}

BOOL CCrossMasterDoc::OpenRecordset (
	CDaoRecordset &Recordset, CString strQuery, long &count)
{
	try
	{
		//      
		Recordset.Open (dbOpenDynaset, strQuery, dbReadOnly);

		//     
		if (!Recordset.IsBOF ())
		{
			Recordset.MoveLast ();
		}
		count = Recordset.GetAbsolutePosition () + 1;
		return TRUE;
	}
	catch (CDaoException* e)
	{
		// 
		if (Recordset.IsOpen ())
			Recordset.Close ();
		count = 0;
		ReportDaoError (e);
		e->Delete ();
		return FALSE;
	}
}

void CCrossMasterDoc::ReportDaoError (CDaoException* e)
{
	CString strErrorMsg;	

	strErrorMsg.Format ("     :\n\"%s\"\n\n"
		"   ,  : %ld\n"
		" : %s\n"
		"\n%s",
		m_strDatabaseName,
		e->m_pErrorInfo->m_lErrorCode,
		e->m_pErrorInfo->m_strSource,
		e->m_pErrorInfo->m_strDescription);
	
	AfxMessageBox (strErrorMsg);
}

BOOL CCrossMasterDoc::LoadTypes (void)
{
	CWaitCursor wc;

	//    
	if (m_dimTypes != NULL)
		return TRUE;

	//     
	CMainFrame* pMainFrame = (CMainFrame*) AfxGetMainWnd();
	ASSERT( pMainFrame );
	ASSERT( pMainFrame->m_wndMainBar );
	CDialogBar &DlgBar = pMainFrame->m_wndMainBar;

	//     ""
	CComboBox* combobox = (CComboBox*) (DlgBar.GetDlgItem (IDC_TYPES_LIST));
	combobox->ResetContent ();

	CListBox* listbox = (CListBox*) (DlgBar.GetDlgItem (IDC_WORDS_LIST));
	listbox->ResetContent ();

	DlgBar.SetDlgItemText (IDC_WORDS_COUNT, "");

	//    " "
	// (     ,     )
	CButton *VBtn = (CButton*) DlgBar.GetDlgItem (IDC_VERTICAL);
	VBtn->SetCheck (1);

	CButton *HBtn = (CButton*) DlgBar.GetDlgItem (IDC_HORIZONTAL);
	HBtn->SetCheck (0);

	CSpinButtonCtrl* spin = (CSpinButtonCtrl*) DlgBar.GetDlgItem (IDC_SPIN_SIZE);
	spin->SetRange (MinWordSize, MaxWordSize);
	spin->SetPos (5);

	ReportStatus ();

	//    
	if (!OpenDatabase ())
		return FALSE;

	//      "TYPES"
	CString strQuery;
	strQuery.Format ("SELECT * FROM %s", m_strTypesTableName);

	//    
	CDaoRecordset Recordset (&m_Database);
	if (!OpenRecordset (Recordset, strQuery, m_nTypesCount))
		return FALSE;
		
	//    
	m_dimTypes = new TypeStruct [m_nTypesCount];
	if (m_dimTypes == NULL)
		return FALSE;

	CString msg;
	COleVariant var1, var2;
	for (long i = 0; i < m_nTypesCount; i++)
	{
		try
		{
			Recordset.SetAbsolutePosition (i);
			
			//  " "     "TYPE"
			var1 = Recordset.GetFieldValue (m_strTypesColumnName);
			if (var1.vt != VT_BSTR)
			{
				combobox->ResetContent ();

				msg.Format ("   %s  %s.\n"
					"  .",
					m_strTypesColumnName, m_strTypesTableName);
				AfxMessageBox (msg);

				Recordset.Close ();
				return FALSE;
			}
			//  " "     "TYPE_KEY"
			var2 = Recordset.GetFieldValue (m_strTypeKeysColumnName);
			if (var2.vt != VT_I4)
			{
				combobox->ResetContent ();

				msg.Format ("   %s  %s.\n"
					"   .",
					m_strTypeKeysColumnName, m_strTypesTableName);
				AfxMessageBox (msg);

				Recordset.Close ();
				return FALSE;
			}
		}
		catch (CDaoException* e)
		{			
			// 
			combobox->ResetContent ();
			ReportDaoError (e);
			Recordset.Close ();
			e->Delete ();			
			return FALSE;
		}

		//   
		combobox->AddString ((LPCSTR) var1.bstrVal);

		//  
		m_dimTypes [i].name = (LPCSTR) var1.bstrVal;
		m_dimTypes [i].id = var2.lVal;
	}
	Recordset.Close ();

	//   
	if (m_nTypeSelect >= m_nTypesCount)
		m_nTypeSelect = 0;
	combobox->SetCurSel (m_nTypeSelect);

	return LoadDictionary ();
}

void CCrossMasterDoc::DeleteTypes (void)
{
	//    
	if (m_dimTypes != NULL)
	{
		delete [] m_dimTypes;
		m_dimTypes = NULL;
	}

	m_nTypesCount = 0;
	m_nDictionaryCount = 0;
}

BOOL CCrossMasterDoc::LoadDictionary (void)
{
	CWaitCursor wc;
	
	if (m_dimTypes == NULL)
		return FALSE;
	
	//    ,  
	if (m_bDictionaryLoaded)
		return TRUE;

	//    
	for (int l = 0; l < MaxWordSize - MinWordSize + 1; l++)
		Crossword.dimDictionarySizesCounter [l] = 0;

	//     
	CDialogBar &DlgBar = ((CMainFrame *) AfxGetMainWnd ())->m_wndMainBar;
	
	CListBox* listbox = (CListBox*) (DlgBar.GetDlgItem (IDC_WORDS_LIST));
	listbox->ResetContent ();

	DlgBar.SetDlgItemText (IDC_WORDS_COUNT, "");

	//      WORDS   
	CString strQuery;
	strQuery.Format ("SELECT * FROM %s WHERE %s = %ld",
		m_strWordsTableName, m_strTypeKeysColumnName,
		m_dimTypes [m_nTypeSelect].id);

	m_nDictionaryCount = 0;

	//    
	CDaoRecordset Recordset (&m_Database);
	if (!OpenRecordset (Recordset, strQuery, m_nDictionaryCount))
		return FALSE;

	//       
	CString strWord;

	CString msg;
	COleVariant var;
	for (long i = 0; i < m_nDictionaryCount; i++)
	{
		try
		{
			Recordset.SetAbsolutePosition (i);
			
			//  " "     "WORD"
			var = Recordset.GetFieldValue (m_strWordsColumnName);
			if (var.vt != VT_BSTR)
			{		
				//   
				listbox->ResetContent ();

				msg.Format ("   %s  %s.\n"
					"  .",
					m_strWordsColumnName, m_strWordsTableName);
				AfxMessageBox (msg);

				Recordset.Close ();
				return FALSE;
			}
		}
		catch (CDaoException* e)
		{			
			// 
			listbox->ResetContent ();
			ReportDaoError (e);
			Recordset.Close ();
			e->Delete ();			
			return FALSE;
		}

		//     
		int Length = strlen ((LPCSTR) var.bstrVal);
		if (Length < MinWordSize || Length > MaxWordSize)
		{
			listbox->ResetContent ();
			
			msg.Format (" \"%s\"    .\n"
				"    %d  %d .",
				(LPCSTR) var.bstrVal, MinWordSize, MaxWordSize);
			AfxMessageBox (msg);
			
			Recordset.Close ();
			return FALSE;
		}
		Crossword.dimDictionarySizesCounter [Length - MinWordSize] ++;
		
		listbox->AddString ((LPCSTR) var.bstrVal);
	}	
	Recordset.Close ();

	//   
	CString strCount;
	
	strCount.Format (" : %ld", m_nDictionaryCount);
	DlgBar.SetDlgItemText (IDC_WORDS_COUNT, strCount);

	m_bDictionaryLoaded = TRUE;

	ReportStatus ();
	return TRUE;
}

void CCrossMasterDoc::ReportStatus (void)
{
	//    
	CDialogBar &DlgBar = ((CMainFrame *) AfxGetMainWnd ())->m_wndMainBar;
	CToolBarCtrl &ToolBar = (CToolBarCtrl &)
		((CMainFrame *) AfxGetMainWnd ())->m_wndToolBar;
	
	CString Text;
	Text.Format (" : %ld", Crossword.CellsListSize);
	DlgBar.GetDlgItem (IDC_CROSSWORD_SIZE)->SetWindowText (Text);

	//   
	ToolBar.EnableButton (ID_CROSSWORD_START, !(m_bBuildInProgress ||
		(Crossword.CellsListSize == 0) || !m_bDictionaryLoaded ||
		(m_dimTypes == NULL)));
	ToolBar.EnableButton (ID_CROSSWORD_STOP,
		m_bBuildInProgress && !m_bTerminateRequest);
	ToolBar.EnableButton (ID_FILE_NEW, !m_bBuildInProgress);
	ToolBar.EnableButton (ID_FILE_OPEN, !m_bBuildInProgress);
	ToolBar.EnableButton (ID_FILE_SAVE, !m_bBuildInProgress);
	ToolBar.EnableButton (ID_FILE_PRINT, !m_bBuildInProgress);

	DlgBar.GetDlgItem (IDC_TYPES_LIST)->EnableWindow (!m_bBuildInProgress);

	UpdateAllViews (NULL);
}

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterDoc serialization

void CCrossMasterDoc::Serialize (CArchive& ar)
{
	if (ar.IsStoring())
	{
		//  
		if (Crossword.Save (ar))
		{
			//   
			try
			{
				ar << m_nTypeSelect;
				return;
			}
			catch (CArchiveException *e)
			{
				//    
				e->Delete ();
			}
		}
		AfxMessageBox ("   .\n"
			"       .");		
	}
	else
	{
		//  
		if (!Crossword.Load (ar))
		{
			AfxMessageBox ("   .\n"
			"    .\n\n"
			"   .");

			ReportStatus ();
			return;
		}

		//   
		long select;
		try
		{
			ar >> select;
		}
		catch (CArchiveException *e)
		{
			//    
			select = m_nTypeSelect;
			e->Delete ();
		}

		if (select < m_nTypesCount && select >= 0 && select != m_nTypeSelect)
		{
			m_nTypeSelect = select;
		
			//         
			CDialogBar &DlgBar = ((CMainFrame *) AfxGetMainWnd ())->m_wndMainBar;
			CComboBox* combobox = (CComboBox*) (DlgBar.GetDlgItem (IDC_TYPES_LIST));
			combobox->SetCurSel (m_nTypeSelect);

			//      
			m_bDictionaryLoaded = FALSE;
			LoadDictionary ();
		}

		ReportStatus ();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterDoc diagnostics

#ifdef _DEBUG
void CCrossMasterDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CCrossMasterDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterDoc commands

void CCrossMasterDoc::OnSelchangeWordsList() 
{
	//     
}

void CCrossMasterDoc::OnSelchangeTypesList() 
{
	ASSERT (m_bBuildInProgress == FALSE);
	
	//     
	CDialogBar &DlgBar = ((CMainFrame *) AfxGetMainWnd ())->m_wndMainBar;

	//   
	CComboBox* combobox = (CComboBox*) (DlgBar.GetDlgItem (IDC_TYPES_LIST));

	//   
	long select = combobox->GetCurSel ();
	if (m_nTypeSelect != select)
	{
		m_nTypeSelect = select;
		
		//     
		SetModifiedFlag ();

		m_bDictionaryLoaded = FALSE;
		LoadDictionary ();
	}
}

void CCrossMasterDoc::OnCrosswordStart() 
{
	ASSERT (m_bBuildInProgress == FALSE);
	ASSERT (Crossword.CellsList != NULL);
	ASSERT (m_dimTypes != NULL);
	ASSERT (m_bDictionaryLoaded);

	CWaitCursor wc;

	m_bBuildResult = FALSE;

	//         
	CString msg;
	for (int l = 0; l < MaxWordSize - MinWordSize + 1; l++)
		if (Crossword.dimDictionarySizesCounter [l] < Crossword.SizesCounter [l])
		{
			msg.Format ("  %d       "
				" .", l + MinWordSize);
			AfxMessageBox (msg);
			return;
		}

	//    
	for (long i = 0; i < Crossword.CellsListSize; i++)
		if (Crossword.CellsList [i].CrossCounter == 0)
		{
			msg.Format ("       "
				".");
			AfxMessageBox (msg);
			return;
		}

	//     
	Crossword.Sort ();
	
	//   
	CDaoRecordset FindRecordset (&m_Database);

	//    , 
	SetModifiedFlag ();

	//   
	m_bBuildInProgress = TRUE;
	m_bTerminateRequest = FALSE;

	ReportStatus ();
	((CMainFrame *) AfxGetMainWnd ())->m_wndStatusBar.SetPaneText (0,
		"  ...");

	sndPlaySound (MAKEINTRESOURCE (IDR_WORK),
		SND_RESOURCE | SND_ASYNC | SND_NOWAIT | SND_NODEFAULT | SND_LOOP);

	//    
	CTime StartTime = CTime::GetCurrentTime();

	m_bBuildResult = BuildCrossword (FindRecordset);
	m_bBuildInProgress = FALSE;
	m_bTerminateRequest = FALSE;

	//    
	CTime EndTime = CTime::GetCurrentTime();
	CTimeSpan TimeSpan = EndTime - StartTime;
	msg.Format (" ,   %d  %d  %d ",
		TimeSpan.GetHours (), TimeSpan.GetMinutes (), TimeSpan.GetSeconds ());
	ReportStatus ();
	((CMainFrame *) AfxGetMainWnd ())->m_wndStatusBar.SetPaneText (0, msg);

	sndPlaySound (MAKEINTRESOURCE (IDR_COMPLETE),
		SND_RESOURCE | SND_ASYNC | SND_NOWAIT | SND_NODEFAULT);
}

void CCrossMasterDoc::OnCrosswordStop()
{
	//    
	m_bTerminateRequest = TRUE;	
}

BOOL CCrossMasterDoc::BuildCrossword (CDaoRecordset& FindRecordset)
{
	ASSERT (Crossword.CellsList != NULL);
	ASSERT (m_dimTypes != NULL);
	ASSERT (m_bDictionaryLoaded == TRUE);

	long CellNumber = 0;
	long WordNumber = 0;
	
	//      
	Crossword.ClearChains ();

	MSG msg;
	do
	{
		//   "" 
		while (PeekMessage (&msg, (HWND) NULL, 0, 0, PM_REMOVE))
		{ 
			TranslateMessage (&msg); 
			DispatchMessage (&msg); 
		}
		if (m_bTerminateRequest)
			//    
			return FALSE;

		WordNumber = FindWordForCell (FindRecordset, CellNumber, WordNumber);
		//  
		if (m_bEnableDrawCell)
			DrawCell (CellNumber);
		if (WordNumber == NullWord)
		{
			//     
			
			//    ,    
			if (CellNumber == 0)
				return FALSE;

			//    
			CellNumber--;
			WordNumber = Crossword.CellsList [CellNumber].NumberOfWord + 1;
		}
		else
		{
			//    

			//  
			CellNumber++;
			WordNumber = 0;
		}
	}
	while (CellNumber < Crossword.CellsListSize);

	return TRUE;
}

long CCrossMasterDoc::FindWordForCell (CDaoRecordset& FindRecordset,
									   long CellNumber, long WordNumber)
{
	ASSERT (CellNumber >= 0);
	ASSERT (CellNumber < Crossword.CellsListSize);
	ASSERT (WordNumber >= 0);

	CCell &cell = Crossword.CellsList [CellNumber];

	//    
	cell.Word = "";
	cell.NumberOfWord = NullWord;

	//  
	CString strTemplate ("");
	int i, n;
	for (i = 0; i < cell.Length; i++)
	{
		n = cell.CrossList [i].CellNumber;
		if (n != NullCell)
		{
			//      
			if (Crossword.CellsList [n].NumberOfWord != NullWord)
			{
				//      ,      
				strTemplate += Crossword.CellsList [n].
					Word.Mid (cell.CrossList [i].CharNumber, 1);
			}
			else
				strTemplate += "?";
		}
		else
			strTemplate += "?";
	}

	//     WORDS     
	CString strQuery;
	strQuery.Format ("SELECT %s FROM %s WHERE %s = %ld AND %s LIKE \"%s\"",
		m_strWordsColumnName, m_strWordsTableName,
		m_strTypeKeysColumnName, m_dimTypes [m_nTypeSelect].id,
		m_strWordsColumnName, strTemplate);

	//   
	long nFindCount;

	//    
	if (!OpenRecordset (FindRecordset, strQuery, nFindCount))
	{
		m_bTerminateRequest = TRUE;
		return NullWord;
	}

	ASSERT (WordNumber <= nFindCount);

	if ((nFindCount == 0) || (WordNumber == nFindCount))
	{
		//   ,   
		FindRecordset.Close ();
		return NullWord;
	}

	CString msg;
	COleVariant var;

	//      
	long index, j;
	for (index = WordNumber; index < nFindCount; index++)
	{
		try
		{
			//     
			FindRecordset.SetAbsolutePosition (index);
				
			//  " "     "WORD"
			var = FindRecordset.GetFieldValue (m_strWordsColumnName);
			if (var.vt != VT_BSTR)
			{
				//    
				FindRecordset.Close ();

				msg.Format ("   %s  %s.\n"
					"  .",
					m_strWordsColumnName, m_strWordsTableName);
				AfxMessageBox (msg);
				m_bTerminateRequest = TRUE;
				return NullWord;
			}
		}
		catch (CDaoException* e)
		{
			// 
			FindRecordset.Close ();
			m_bTerminateRequest = TRUE;
			ReportDaoError (e);
			e->Delete ();			
			return NullWord;
		}

		//        
		int Length = strlen ((LPCSTR) var.bstrVal);
		for (j = 0; j < Crossword.CellsListSize; j++)
			//       ,  
			if (Crossword.CellsList [j].Length == Length)
				//   -
				if (Crossword.CellsList [j].Word == (LPCSTR) var.bstrVal)
					break;
		
		if (j == Crossword.CellsListSize)
			//    
			break;
	}
	FindRecordset.Close ();
	
	if (index == nFindCount)
		//      
		return NullWord;

	//   
	cell.Word = (LPCSTR) var.bstrVal;
	cell.NumberOfWord = index;

	return index;
}

void CCrossMasterDoc::OnUpdateCrosswordStart(CCmdUI* pCmdUI) 
{
	//       OK
	pCmdUI->Enable (!(m_bBuildInProgress ||
		(Crossword.CellsListSize == 0) || !m_bDictionaryLoaded ||
		(m_dimTypes == NULL)));	
}

void CCrossMasterDoc::OnUpdateCrosswordStop(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (m_bBuildInProgress && !m_bTerminateRequest);	
}

void CCrossMasterDoc::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (!m_bBuildInProgress);	
}

void CCrossMasterDoc::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (!m_bBuildInProgress);	
}

void CCrossMasterDoc::OnUpdateFilePrint(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (!m_bBuildInProgress);	
}

void CCrossMasterDoc::OnUpdateFilePrintPreview(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (!m_bBuildInProgress);	
}

void CCrossMasterDoc::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (!m_bBuildInProgress);	
}

void CCrossMasterDoc::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (!m_bBuildInProgress);	
}

void CCrossMasterDoc::OnUpdateAppExit(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (!m_bBuildInProgress);	
}

void CCrossMasterDoc::OnUpdateFileMruFile1(CCmdUI* pCmdUI) 
{
	pCmdUI->ContinueRouting ();
	pCmdUI->Enable (!m_bBuildInProgress);	
}

void CCrossMasterDoc::OnUpdateFileAccess(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable (!m_bBuildInProgress);	
}

BOOL CCrossMasterDoc::CanCloseFrame(CFrameWnd* pFrame) 
{
	if (m_bBuildInProgress)
	{
		AfxMessageBox ("     .\n"
			"   .");
		return FALSE;
	}
	
	return CDocument::CanCloseFrame(pFrame);
}

void CCrossMasterDoc::OnFileAccess() 
{
	CWaitCursor wc;

	//    ,  MS Access
	ShellExecute (AfxGetMainWnd ()->m_hWnd, "open", m_strDatabaseName, NULL, "",
		SW_SHOWNORMAL);
}

void CCrossMasterDoc::DrawCell (long index, CDC *pDC)
{
	ASSERT (index >= 0);
	ASSERT (index < Crossword.CellsListSize);

	CCell &cell = Crossword.CellsList [index];
	
	CDC *pCellDC;
	CView* pView = NULL;
	if (pDC == NULL)
	{
		//     
		POSITION pos = GetFirstViewPosition ();
		pView = GetNextView (pos);
		pCellDC = pView->GetDC ();

		pCellDC->SetBkMode (TRANSPARENT);
		pCellDC->SelectStockObject (ANSI_VAR_FONT);
	}
	else
		//   
		pCellDC = pDC;
	
	CRect rectCell;
	int x, y;
	for (int j = 0; j < cell.Length; j++)
	{
		if (cell.CrossList [j].CellNumber != NullCell && j != 0)
			if (cell.CrossList [j].CharNumber == 0)
				continue;
		
		x = cell.X * CellSize + CellSize;
		y = cell.Y * CellSize + CellSize;
		if (cell.Type == Vertical)
			y += CellSize * j;
		else
			x += CellSize * j;
		rectCell.SetRect (x + 1, y + 1,
			x + CellSize - 1, y + CellSize - 1);

		pCellDC->Rectangle (&rectCell);	

		if (j == 0)
		{
			//     
			rectCell.DeflateRect (1, 1, 1, 1);
			pCellDC->Rectangle (&rectCell);
		}

		if (cell.Word.GetLength () > 0)
			pCellDC->DrawText (cell.Word.Mid (j, 1), &rectCell,
				DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_NOCLIP);
	}

	//  ,   
	if ( pDC == NULL && pView )
		pView->ReleaseDC (pCellDC);
}

void CCrossMasterDoc::OnViewCell() 
{
	m_bEnableDrawCell = !m_bEnableDrawCell;
}

void CCrossMasterDoc::OnUpdateViewCell (CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck (m_bEnableDrawCell);	
}
