// CrossMasterView.cpp : implementation of the CCrossMasterView class
//

#include "stdafx.h"
#include "CrossMaster.h"

#include "MainFrm.h"

#include "CrossMasterDoc.h"
#include "CrossMasterView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const COLORREF CursorColor = RGB (0, 0, 255);
const COLORREF DeniedColor = RGB (255, 0, 0);
const COLORREF BackgroundColor = RGB (255, 255, 255);

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterView

IMPLEMENT_DYNCREATE(CCrossMasterView, CScrollView)

BEGIN_MESSAGE_MAP(CCrossMasterView, CScrollView)
	//{{AFX_MSG_MAP(CCrossMasterView)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_ERASEBKGND()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterView construction/destruction

CCrossMasterView::CCrossMasterView()
{
	//   
	CursorRect.SetRect (0, 0, 0, 0);
}

CCrossMasterView::~CCrossMasterView()
{
}

BOOL CCrossMasterView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterView drawing

void CCrossMasterView::OnDraw(CDC* pDC)
{
	CCrossMasterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	//   
	CRect rectClient;
	CRect rectCell;
	CString stringLabel;

	pDC->SetBkMode (TRANSPARENT);
	pDC->SelectStockObject (NULL_BRUSH);
	pDC->SelectStockObject (ANSI_VAR_FONT);
	
	GetClientRect (&rectClient);

	//   
	pDC->MoveTo (rectClient.left, rectClient.top + CellSize - 1);
	pDC->LineTo (rectClient.right, rectClient.top + CellSize - 1); 
	
	//   
	pDC->MoveTo (rectClient.left + CellSize - 1, rectClient.top);
	pDC->LineTo (rectClient.left + CellSize - 1, rectClient.bottom); 

	//   
	for (int i = 0; i < rectClient.Width () / CellSize; i++)
	{
		rectCell.SetRect (rectClient.left + CellSize * (i + 1),
			rectClient.top,
			rectClient.left + CellSize * (i + 2),
			rectClient.top + CellSize - 1);
		stringLabel.Format ("%d", i);
		pDC->DrawText (stringLabel, &rectCell,
			DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_NOCLIP);
	}

	//   
	for (int i = 0; i < rectClient.Height () / CellSize; i++)
	{
		rectCell.SetRect (rectClient.left,
			rectClient.top + CellSize * (i + 1),
			rectClient.left + CellSize - 1,
			rectClient.top + CellSize * (i + 2));
		stringLabel.Format ("%d", i);
		pDC->DrawText (stringLabel, &rectCell,
			DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_NOCLIP);
	}

	if (pDoc->Crossword.CellsList != NULL)
		//   (  )
		for (long index = 0; index < pDoc->Crossword.CellsListSize; index++)
			pDoc->DrawCell (index, pDC);

	//   (..      )
	MoveCursor (CursorRect.left, CursorRect.top);
}

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterView printing

BOOL CCrossMasterView::OnPreparePrinting(CPrintInfo* pInfo)
{
	//   
	pInfo->SetMinPage (1);
	pInfo->SetMaxPage (1);

	//    
	pInfo->m_strPageDesc = (" %u\n %u-%u\n");

	return DoPreparePrinting(pInfo);
}

void CCrossMasterView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

#pragma warning (disable:4100) 
void CCrossMasterView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	pDC->TextOut (0, 0, "    .");
	
	//  
	//CScrollView::OnPrint(pDC, pInfo);
}

void CCrossMasterView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterView diagnostics

#ifdef _DEBUG
void CCrossMasterView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CCrossMasterView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CCrossMasterDoc* CCrossMasterView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCrossMasterDoc)));
	return (CCrossMasterDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCrossMasterView message handlers

void CCrossMasterView::OnMouseMove (UINT nFlags, CPoint point) 
{
	if ((point.x >= CellSize) && (point.y >= CellSize))
	{
		int x = point.x / CellSize - 1;
		int y = point.y / CellSize - 1;
		
		if (x != CursorRect.left || y != CursorRect.top)
			MoveCursor (x, y);
	}

	CScrollView::OnMouseMove(nFlags, point);
}

void CCrossMasterView::MoveCursor (int x, int y) 
{
	CCrossMasterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CDialogBar &DlgBar = ((CMainFrame *) AfxGetMainWnd ())->m_wndMainBar;
	CDC *pDC = GetDC ();
	
	//           
	int Length = DlgBar.GetDlgItemInt (IDC_EDIT_SIZE);
	if (Length < MinWordSize)
	{
		Length = MinWordSize;
		DlgBar.SetDlgItemInt (IDC_EDIT_SIZE, Length);
		DlgBar.UpdateWindow ();
	}
	else
		if (Length > MaxWordSize)
		{
			Length = MaxWordSize;
			DlgBar.SetDlgItemInt (IDC_EDIT_SIZE, Length);
			DlgBar.UpdateWindow ();
		}
	
	//     (TRUE -  )
	BOOL Orientation = ((CButton *) (DlgBar.GetDlgItem (IDC_VERTICAL)))->
		GetCheck () == BST_CHECKED;
	
	//    (..   )
	DrawCursor (CursorRect, pDC, BackgroundColor);	

	//     
	CursorRect.left = x;
	CursorRect.top = y;
	CursorRect.right = Orientation ? 0 : Length;
	CursorRect.bottom = Orientation ? Length : 0;

	//   
	DrawCursor (CursorRect, pDC,
		(pDoc->Crossword.TestAdd (Orientation ? Vertical : Horizontal, x, y, Length) ||
		pDoc->Crossword.TestRemove (Orientation ? Vertical : Horizontal, x, y)) ?
		CursorColor : DeniedColor);	
	
	ReleaseDC (pDC);
}

void CCrossMasterView::DrawCursor (CRect &rectCursor, CDC *pDC, COLORREF rgb) 
{
	//     ,   
	int i = 0;
	if (rectCursor.right)
		for (; i < rectCursor.right; i++)
			pDC->Draw3dRect ((rectCursor.left + 1 + i) * CellSize,
				(rectCursor.top + 1) * CellSize, CellSize, CellSize, rgb, rgb);
	else
		for (; i < rectCursor.bottom; i++)
			pDC->Draw3dRect ((rectCursor.left + 1) * CellSize,
				(rectCursor.top + 1 + i) * CellSize, CellSize, CellSize, rgb, rgb);
}

void CCrossMasterView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CCrossMasterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (pDoc->m_bBuildInProgress)
		//      
		return;

	if ((point.x >= CellSize) && (point.y >= CellSize))
	{
		MoveCursor (point.x / CellSize - 1, point.y / CellSize - 1);

		int Type = CursorRect.bottom ? Vertical : Horizontal;
		int Length = CursorRect.bottom + CursorRect.right;

		if (!pDoc->Crossword.Add (Type,	CursorRect.left, CursorRect.top, Length))
			if (!pDoc->Crossword.Remove (Type, CursorRect.left, CursorRect.top))
			{
				sndPlaySound (MAKEINTRESOURCE (IDR_NOCLICK),
					SND_RESOURCE | SND_ASYNC | SND_NOWAIT | SND_NODEFAULT);
				return;
			}
	
		sndPlaySound (MAKEINTRESOURCE (IDR_CLICK),
			SND_RESOURCE | SND_ASYNC | SND_NOWAIT | SND_NODEFAULT);

		pDoc->SetModifiedFlag ();
		pDoc->ReportStatus ();
		return;
	}
	
	CScrollView::OnLButtonDown(nFlags, point);
}

BOOL CCrossMasterView::OnEraseBkgnd(CDC* pDC) 
{
	//  
	CRect rectClient;
	GetClientRect (rectClient);

	pDC->FillSolidRect (rectClient, BackgroundColor);
	
	return TRUE;
}

void CCrossMasterView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CCrossMasterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CRect rectCrossword;
	pDoc->Crossword.GetRect (rectCrossword);

	//    20 x 20  
	SetScrollSizes (MM_TEXT, CSize (
		(rectCrossword.right + 1) * CellSize,
		(rectCrossword.bottom + 1) * CellSize),
		CSize (21 * CellSize, 21 * CellSize),  CSize (CellSize, CellSize));

	CScrollView::OnUpdate (pSender, lHint, pHint);
}

void CCrossMasterView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	//CScrollView::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CCrossMasterView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	//CScrollView::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CCrossMasterView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	CCrossMasterDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->LoadTypes();
}
